/* SPDX-License-Identifier: GPL-2.0 */
#ifndef _ASM_SW64_CHIP2_IO_H
#define _ASM_SW64_CHIP2_IO_H

#include <asm/platform.h>

#define MIU_IO_BASE		(0x5UL << 36)
#define IPU_IO_BASE		(0x6UL << 36)

#define IO_BASE			(0x1UL << 47)
#define PCI_BASE		(0X4UL << 36)
#define PCI0_IO_BASE		(0x4UL << 36)
#define PCI1_IO_BASE		(0x7UL << 36)
#define PCI_IOR0_BASE		(0x4UL << 32)
#define PCI_IOR1_BASE		(0x5UL << 32)
#define PCI_RC_CFG		(0x8UL << 32)
#define PCI_EP_CFG		(0xaUL << 32)
#define PCI_LEGACY_IO		(0x6UL << 32)
#define SW64_PCI_IO_BASE(m, n)	(IO_BASE | PCI0_IO_BASE | ((0x3UL << 36) * (n)))
#define SW64_IO_BASE(x)		IO_BASE

#define PCI_LEGACY_IO_SIZE	0xf00000UL
#define PCI_32BIT_MEMIO		0xe0000000UL
#define PCI_32BIT_MEMIO_SIZE	0x20000000UL

#ifndef CONFIG_SICH_VIDEO
#define PCI_64BIT_MEMIO		0x3c0000000UL
#define PCI_64BIT_MEMIO_SIZE	0x40000000UL
#else
#define PCI_64BIT_MEMIO		0x200000000UL
#define PCI_64BIT_MEMIO_SIZE	0x200000000UL
#endif

#define PCI0_BUS		0

#define MAX_NR_RCS		0x2
#define IO_MARK_BIT		47

enum MIU_REG {
	CG_ONLINE		= MIU_IO_BASE | (0x00000000) << 7,
	CG0_FAULT_STAT		= MIU_IO_BASE | (0x00000001) << 7,
	CG1_FAULT_STAT		= MIU_IO_BASE | (0x00000002) << 7,
	CG2_FAULT_STAT		= MIU_IO_BASE | (0x00000003) << 7,
	CG3_FAULT_STAT		= MIU_IO_BASE | (0x00000004) << 7,
	CG0_FAULT_EN		= MIU_IO_BASE | (0x00000005) << 7,
	CG1_FAULT_EN		= MIU_IO_BASE | (0x00000006) << 7,
	CG2_FAULT_EN		= MIU_IO_BASE | (0x00000007) << 7,
	CG3_FAULT_EN		= MIU_IO_BASE | (0x00000008) << 7,
	CG0_FAULT_INT_EN	= MIU_IO_BASE | (0x00000009) << 7,
	CG1_FAULT_INT_EN	= MIU_IO_BASE | (0x0000000a) << 7,
	CG2_FAULT_INT_EN	= MIU_IO_BASE | (0x0000000b) << 7,
	CG3_FAULT_INT_EN	= MIU_IO_BASE | (0x0000000c) << 7,
	INIT_CTL		= MIU_IO_BASE | (0x0000000d) << 7,
	CORE_ONLINE		= MIU_IO_BASE | (0x0000000f) << 7,
	CPUID			= MIU_IO_BASE | (0x00000012) << 7,
	LONG_TIME		= MIU_IO_BASE | (0x00000023) << 7,
	LONG_PRESET		= MIU_IO_BASE | (0x00000024) << 7,
	IO_START		= MIU_IO_BASE | (0x00000026) << 7,
	SI_FAULT_STAT		= MIU_IO_BASE | (0x00000062) << 7,
	SI_FAULT_EN		= MIU_IO_BASE | (0x00000063) << 7,
	SI_FAULT_INT_EN		= MIU_IO_BASE | (0x00000064) << 7,
	MC_ONLINE		= MIU_IO_BASE | (0x0000006f) << 7,
	MC_CONFIG		= MIU_IO_BASE | (0x00000092) << 7
};

enum IPU_REG {
	SHORT_CLK_START		= IPU_IO_BASE | (0x000000004) << 7,
	SHORT_CLK_MSK		= IPU_IO_BASE | (0x000000005) << 7,
	INTEN			= IPU_IO_BASE | (0x000000008) << 7,
	SYSERR_NMI_INT_DST	= IPU_IO_BASE | (0x000000017) << 7
};

/* PIU IOR0 */
enum {
	PIUCONFIG0 =		0x0UL,
	EPDMABAR =		0x80UL,
	IOMMUSEGITEM0 =		0x100UL,
	IOMMUEXCPT_CTRL =	0x2100UL,
	MSIADDR =		0x2180UL,
	MSICONFIG0 =		0x2200UL,
	INTACONFIG =		0xa200UL,
	INTBCONFIG =		0xa280UL,
	INTCCONFIG =		0xa300UL,
	INTDCONFIG =		0xa380UL,
	AERERRINTCONFIG =	0xa400UL,
	AERERRMSICONFIG =	0xa480UL,
	PMEINTCONFIG =		0xa500UL,
	PMEMSICONFIG =		0xa580UL,
	HPINTCONFIG =		0xa600UL,
	HPMSICONFIG =		0xa680UL,
	DTBASEADDR =		0xb000UL,
	DTLB_FLUSHALL =		0xb080UL,
	DTLB_FLUSHDEV =		0xb100UL,
	PTLB_FLUSHALL =		0xb180UL,
	PTLB_FLUSHDEV =		0xb200UL,
	PTLB_FLUSHVADDR =	0xb280UL,
	PCACHE_FLUSHALL =	0xb300UL,
	PCACHE_FLUSHDEV =	0xb380UL,
	PCACHE_FLUSHPADDR =	0xb400UL,
	TIMEOUT_CONFIG =	0xb480UL,
	IOMMUEXCPT_STATUS =	0xb500UL,
	IOMMUPAGE_PADDR1 =	0xb580UL,
	IOMMUPAGE_PADDR2 =	0xb600UL,
	IOMMUPAGE_PADDR3 =	0xb680UL,
	PTLB_ACCESS =		0xb700UL,
	PTLB_ITEM_TAG =		0xb780UL,
	PTLB_ITEM_DATA =	0xb800UL,
	PCACHE_ACCESS =		0xb880UL,
	PCACHE_ITEM_TAG =	0xb900UL,
	PCACHE_ITEM_DATA0 =	0xb980UL,
};

/* PIU IOR1 */
enum {
	PIUCONFIG1 =		0x0UL,
	ERRENABLE =		0x880UL,
	RCCONTROL =		0xb00UL,
	RCDEBUGINF0 =		0xc00UL,
	RCDEBUGINF1 =		0xc80UL,
	DCACONTROL =		0x1a00UL,
	DEVICEID0 =		0x1a80UL,
};

/* RC */
enum {
	RC_COMMAND =		0x80UL,
	RC_PRIMARY_BUS =	0x300UL,
	RC_MSI_CONTROL =	0xa00UL,
	RC_EXP_DEVCAP =		0xe80UL,
	RC_EXP_DEVCTL =		0xf00UL,
	RC_LINK_STAT =		0x1000UL,
	RC_CONTROL =		0X1180UL,
	RC_STATUS =		0X1200UL,
	RC_EXP_DEVCTL2 =	0x1300UL,
	RC_LINK_CONTROL2 =	0x1400UL,
	RC_PORT_LINK_CTL =	0xe200UL,
};

#define PCIE0_MSI_INDX_BASE            (IO_BASE | IPU_IO_BASE | (0x100UL << 7))
#define PCIE1_MSI_INDX_BASE            (IO_BASE | IPU_IO_BASE | (0x200UL << 7))

#define write_pcie_msi_index(reg, x, val)				\
	({								\
		void __iomem *addr;					\
		addr = __va(PCIE##x##_MSI_INDX_BASE + ((reg) << 7));	\
		writeq(val, addr);					\
	})

#define write_pcie0_msi_index(reg, val)		\
	write_pcie_msi_index((reg), 0, (val))
#define write_pcie1_msi_index(reg, val)		\
	write_pcie_msi_index((reg), 1, (val))

#endif   /* _ASM_SW64_CHIP2_IO_H */
