/*
 * Copyright (C) 2023 nfs
 * Copyright 2023 Guosheng Guo <guosheng1@nfschina.com>
 */


#ifndef __SM_DEVCTL_H
#define __SM_DEVCTL_H

#include <linux/kdev_t.h>
#include <linux/usb/ch9.h>
#include <linux/net.h>
#include <uapi/linux/ppdev.h>
#include <asm-generic/compat.h>

/* define device permission control, reference 
 * uapi/linux/sockios.h */
#define SM_DEVCTL_PERM		0x89b2
#define SM_DEVCTL_VIDPID_PERM	0x89b3
#define SM_DEVCTL_VM_PERM	0x89b4
#define SM_PKG_PROTECT_PERM	0x89b5
#define SM_FILE_PROTECT_PERM	0x89b6	/* tempory for file protect */

#define SM_DEVCTL_GETPERM		0xf7c2
#define SM_DEVCTL_VIDPID_GETPERM	0xf7c3
#define SM_DEVCTL_VM_GETPERM	0xf7c4
#define SM_PKG_PROTECT_GETPERM	0xf7c5
#define SM_FILE_PROTECT_GETPERM	0xf7c6	/* tempory for file protect */
#define SM_NICNAME_GETPERM	0xf7c7	

#define VM_PATH_LEN		1024
#define PKG_NAME_LEN		256

enum interface_type {
	SM_INF_SERIAL = 1, 	/* 串口 */
	SM_INF_PARALLEL, 	/* 并口 */
	SM_INF_1394, 		/* 1394 接口 */
	SM_INF_PCMCIA, 		/* PCMCIA 接口 */
	SM_INF_TYPEC,		/* Type-C 接口 */
	SM_INF_HDMI,		/* HDMI 接口 */
	SM_INF_DISPLAYPORT,	/* DisplayPort 接口 */
	INF_MAX
};

enum dev_type {
    	SM_DEV_BLUETOOTH = 1, 	/* 蓝牙 */
    	SM_DEV_INFRARED,  	/* 红外 */
    	SM_DEV_MODEM,  		/* modem 设备*/
    	SM_DEV_STORAGE_DRIVE, 	/* 存储驱动器(SD、TF 卡) */
    	SM_DEV_SCSI_RAID,  	/* SCSI 及 RAID 控制器 */
    	SM_DEV_WINCE,  		/* wince 设备(安卓手机、pad 等) */
    	SM_DEV_FLOPPY_DRIVE,  	/* 软驱 */
    	SM_DEV_IMAGE,  		/* 图像设备(包括摄像头、
				   数码相机、扫描仪) */
    	SM_DEV_MULTI_SERIAL_CARD, 	/* 多串口卡*/
	SM_DEV_ETHERNET,	/* 有线网卡 */
	SM_DEV_WIRELESS,	/* 无线网卡 */
    	SM_DEV_PRINTER,  	/* 打印机*/
	SM_DEV_PRINTER_LOCAL,	/* 直连打印机 */
	SM_DEV_PRINTER_NET,	/* 网络打印机 */
    	SM_DEV_CDROM,  		/* 刻录*/
	DEV_VOLUMESHADOW,	/* 卷影设备 */
	DEV_TAPE,		/* 磁带机 */
    	DEV_MAX
};

enum usb_subtype {
	SM_USB_CLASS_PER_INTERFACE=1, 	/* 00h,保留分类 */
	SM_USB_CLASS_AUDIO, 		/* 01h,音频设备 */
	SM_USB_CLASS_COMM, 		/* 02h,通信及通信类 */ 
					/*   控制设备 */
	SM_USB_CLASS_HID, 		/* 03h,人体工学设备 */
	SM_USB_CLASS_PHYSICAL, 		/* 05h,物理设备 */
	SM_USB_CLASS_STILL_IMAGE, 	/* 06h,图像 */
	SM_USB_CLASS_PRINTER, 		/* 07h,打印设备 */
	SM_USB_CLASS_MASS_STORAGE, 	/* 08h,大容量存储 */
	SM_USB_CLASS_HUB, 		/* 09h,集线器 */
	SM_USB_CLASS_CDC_DATA, 		/* 0Ah,通信类数据设备 */
	SM_USB_CLASS_CSCID, 		/* 0Bh,智能卡 */
	SM_USB_CLASS_CONTENT_SEC, 	/* 0Dh,内容安全 */
	SM_USB_CLASS_VIDEO, 		/* 0Eh,视频 */
	SM_USB_CLASS_WIRELESS_CONTROLLER, 	/* E0h,无线控制器 */
	SM_USB_CLASS_MISC, 		/* EFh,其他设备 */
	SM_USB_CLASS_APP_SPEC, 		/* FEh,应用程序 */
	SM_USB_CLASS_VENDOR_SPEC, 	/* FFh,厂家自定义 */
	SM_USB_CLASS_PERSONAL_HEALTH,	/* 0F,个人健康 */
	SM_USB_CLASS_DIAGNOSTIC,       	/* DC，诊断设备 */
	USB_MAX
};
/* 设置当前策略的接口 */
extern long sm_devctl_perm_ioctl(struct socket *sock, 
		unsigned int cmd, compat_ulong_t __user *argp);
extern long sm_devctl_vidpid_perm_ioctl(struct socket *sock, 
		unsigned int cmd, compat_ulong_t __user *argp);
extern long sm_devctl_vm_perm_ioctl(struct socket *sock, 
		unsigned int cmd, compat_ulong_t __user *argp);
/* 获取当前生效的策略用的接口 */
extern long sm_devctl_get_perm_ioctl(struct socket *sock, 
		unsigned int cmd, compat_ulong_t __user *argp);
extern long sm_devctl_vidpid_get_perm_ioctl(struct socket *sock, 
		unsigned int cmd, compat_ulong_t __user *argp);
extern long sm_devctl_nicname_get_perm_ioctl(struct socket *sock, 
		unsigned int cmd, compat_ulong_t __user *argp);
extern long sm_devctl_vm_get_perm_ioctl(struct socket *sock, 
		unsigned int cmd, compat_ulong_t __user *argp);
/* 已经弃用的内核中包防卸载接口 */
extern long sm_pkg_protect_ioctl(struct socket *sock, 
		unsigned int cmd, compat_ulong_t __user *argp);
/* file protect for tempory */
extern long sm_file_protect_ioctl(struct socket *sock, 
		unsigned int cmd, compat_ulong_t __user *argp);
extern int dev_perm_chk(struct device *dev);
extern void dev_perm_clean(const char * dev_name);
extern int fs_perm_chk(struct inode *inode);
extern int vm_perm_chk(const char *vm_path);
extern int pkg_perm_chk(const char *pkg_name);
/* file protect for tempory */
extern int file_perm_chk(const char *process_name);
extern void burning_setup(int burn);

extern void class_addr_add(struct class *cls);
extern void class_addr_del(struct class *cls);
#endif /* __SM_DEVCTL_H */
